/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include <dawn/dawn.h>
#include "card.h"

/**
 * Let a player bet chips into the pot.
 * 
 * @param poker Poker game instance.
 * @param player Poker player instance.
 * @param chips Chips to bet.
 */
void pokerPlayerBet(poker_t *poker, pokerplayer_t *player, int32_t chips);

/**
 * Returns true if the player is still alive and in the current game/
 * Defined as: Not out, not folded.
 * 
 * @param player Player to check
 * @returns True if alive/in teh current game.
 */
bool pokerPlayerIsAlive(pokerplayer_t *player);

/**
 * Returns true if the player provided is human or not.
 * 
 * @param poker Poker game instance.
 * @param player Player instance.
 * @returns True if the player is human.
 */
bool pokerPlayerIsHuman(poker_t *poker, pokerplayer_t *player);

/**
 * Deal a card to a player.
 * 
 * @param poker Poker game instance.
 * @param player Poker player to deal to.
 */
void pokerPlayerDeal(poker_t *poker, pokerplayer_t *player);

/**
 * Deal card(s) to every active player.
 * 
 * @param poker Poker game instance.
 * @param count Count of cards to deal.
 */
void pokerPlayerDealAll(poker_t *poker, uint8_t count);

/**
 * Returns the full hand for a given player including the best cards on the 
 * bench.
 * 
 * @param poker Poker game instance.
 * @param player Poker player game instance.
 * @param cards Array of at least 7 length to store the array.
 */
void pokerPlayerGetFullHand(poker_t *poker,pokerplayer_t *player,card_t *cards);

/**
 * Calculates and returns the winning state for a given player
 * 
 * @param poker Poker game instance.
 * @param player Player game instance.
 * @param winning Pointer to the poker winning to fill out.
 * @return The winning state for this player. 
 */
void pokerPlayerGetWinning(
  poker_t *poker, pokerplayer_t *player, pokerwinning_t *winning
);

/**
 * Fills the remaining cards for a given poker player winning hand.
 * @param winning Pointer to the poker winning to fill out.
 */
void pokerPlayerWinningFillRemaining(pokerwinning_t *winning);

/**
 * Compares two winning sets. The returned card is the kicker if the LEFT side
 * is the winner. If LEFT is not a winner then 0xFF will be returned.
 * 
 * @param left Left winning set.
 * @param right Right winning set.
 * @return The kicker card from left's hand or 0xFF if left is not the winner.
 */
card_t pokerPlayerCompareWinning(pokerwinning_t *left, pokerwinning_t *right);

/**
 * Determines the winning player for the game. Values will be stored back into
 * the poker winning state.
 * 
 * @param poker Poker game instance.
 */
void pokerPlayerWinnerDetermine(poker_t *poker);